@@ -24,6 +24,7 @@
 #include <stdlib.h>
 #include <sys/types.h>
 #include <pwd.h>
+#include <utmp.h>
 
 #include <glib.h>
 #include <glib/gi18n.h>
@@ -33,6 +34,8 @@
 #include "um-user-manager.h"
 #include "um-utils.h"
 
+#define MAXNAMELEN  (UT_NAMESIZE - 1)
+
 struct _UmAccountDialog {
         GtkWidget *dialog;
         GtkWidget *shortname_combo;
@@ -128,46 +131,6 @@ is_shortname_used (const gchar *shortname)
         return pwent != NULL;
 }
 
-static gboolean
-query_tooltip (GtkWidget  *widget,
-               gint        x,
-               gint        y,
-               gboolean    keyboard_mode,
-               GtkTooltip *tooltip,
-               gpointer    user_data)
-{
-        gchar *tip;
-
-        if (GTK_ENTRY_ICON_SECONDARY == gtk_entry_get_icon_at_pos (GTK_ENTRY (widget), x, y)) {
-                tip = gtk_entry_get_icon_tooltip_text (GTK_ENTRY (widget),
-                                                       GTK_ENTRY_ICON_SECONDARY);
-                gtk_tooltip_set_text (tooltip, tip);
-                g_free (tip);
-
-                return TRUE;
-        }
-        else {
-                return FALSE;
-        }
-}
-
-static void
-icon_released (GtkEntry             *entry,
-              GtkEntryIconPosition  pos,
-              GdkEvent             *event,
-              gpointer              user_data)
-{
-        GtkSettings *settings;
-        gint timeout;
-
-        settings = gtk_widget_get_settings (GTK_WIDGET (entry));
-
-        g_object_get (settings, "gtk-tooltip-timeout", &timeout, NULL);
-        g_object_set (settings, "gtk-tooltip-timeout", 1, NULL);
-        gtk_tooltip_trigger_tooltip_query (gtk_widget_get_display (GTK_WIDGET (entry)));
-        g_object_set (settings, "gtk-tooltip-timeout", timeout, NULL);
-}
-
 static void
 shortname_changed (GtkComboBox     *combo,
                    UmAccountDialog *um)
@@ -175,6 +138,7 @@ shortname_changed (GtkComboBox     *combo,
         gboolean in_use;
         gboolean empty;
         gboolean valid;
+        gboolean toolong;
         const gchar *shortname;
         const gchar *c;
         gchar *tip;
@@ -183,64 +147,53 @@ shortname_changed (GtkComboBox     *combo,
         shortname = gtk_combo_box_get_active_text (combo);
 
         in_use = is_shortname_used (shortname);
-        empty = strlen (shortname) == 0;
+        empty = shortname[0] == 0;
+        toolong = strlen (shortname) > MAXNAMELEN;
         valid = TRUE;
 
-        if (!in_use && !empty) {
+        if (!in_use && !empty && !toolong) {
                 /* First char must be a letter, and it must only composed
                  * of ASCII letters, digits, and a '.', '-', '_'
                  */
                 for (c = shortname; *c; c++) {
-                        if ((c == shortname && !g_ascii_islower (*c)) ||
-                            !(g_ascii_isdigit (*c) || g_ascii_islower (*c) ||
-                              *c == '.' || *c == '-' || *c == '_' )) {
-                                valid = FALSE;
-                        }
+                        if (! ((*c >= 'a' && *c <= 'z') ||
+                               (*c >= 'A' && *c <= 'Z') ||
+                               (*c >= '0' && *c <= '9') ||
+                               (*c == '_') || (*c == '.') ||
+                               (*c == '-' && c != shortname)))
+                           valid = FALSE;
                 }
         }
 
-        um->valid_shortname = !empty && !in_use && valid;
+        um->valid_shortname = !empty && !in_use && !toolong && valid;
         gtk_widget_set_sensitive (um->ok_button, um->valid_name && um->valid_shortname);
 
         entry = gtk_bin_get_child (GTK_BIN (combo));
 
-        if (!empty && (in_use || !valid)) {
-                gtk_entry_set_icon_from_stock (GTK_ENTRY (entry),
-                                               GTK_ENTRY_ICON_SECONDARY,
-                                               GTK_STOCK_DIALOG_ERROR);
-                gtk_entry_set_icon_activatable (GTK_ENTRY (entry),
-                                                GTK_ENTRY_ICON_SECONDARY,
-                                                TRUE);
-                g_signal_connect (entry, "icon-release",
-                                  G_CALLBACK (icon_released), FALSE);
-                g_signal_connect (entry, "query-tooltip",
-                                  G_CALLBACK (query_tooltip), NULL);
-
+        if (!empty && (in_use || toolong || !valid)) {
                 if (in_use) {
-                        tip = g_strdup_printf (_("A user with the short name '%s' already exists."),
+                        tip = g_strdup_printf (_("A user with the short name '%s' already exists"),
                                                shortname);
                 }
-                else if (!g_ascii_islower (shortname[0])) {
-                        tip = g_strdup (_("The short name must start with a lowercase letter."));
+                else if (toolong) {
+                        tip = g_strdup_printf (_("The short name is too long"));
+                }
+                else if (shortname[0] == '-') {
+                        tip = g_strdup (_("The short name cannot start with a '-'"));
                 }
                 else {
                         tip = g_strdup (_("The short name must consist of:\n"
-                                          " \xe2\x9e\xa3 lower case letters from the English alphabet\n"
+                                          " \xe2\x9e\xa3 letters from the English alphabet\n"
                                           " \xe2\x9e\xa3 digits\n"
                                           " \xe2\x9e\xa3 any of the characters '.', '-' and '_'"));
                 }
 
-                g_object_set (entry, "has-tooltip", TRUE, NULL);
-                gtk_entry_set_icon_tooltip_text (GTK_ENTRY (entry),
-                                                 GTK_ENTRY_ICON_SECONDARY,
-                                                 tip);
+                set_entry_validation_error (GTK_ENTRY (entry), tip);
+
                 g_free (tip);
         }
         else {
-               g_object_set (entry, "has-tooltip", FALSE, NULL);
-               gtk_entry_set_icon_from_pixbuf (GTK_ENTRY (entry),
-                                               GTK_ENTRY_ICON_SECONDARY,
-                                               NULL);
+                clear_entry_validation_error (GTK_ENTRY (entry));
         }
 }
 
